/*global define, google*/
define('de_epages/externalcontent/gadgetdialog/maps-view', [
    'jquery',
    'ep',
    './constants',
    './config-view',
    '$tmpl!./maps-view',
    '$tmpl!./maps-view-error-message',
    '$tmpl!./maps-view-key-in-use',
    '$dict!../dictionary',

    '../ui/maps' // The gadget to configure.
], function ($, ep, CONSTANTS, ConfigView, template, templateError, templateKeyInUse, dict) {

    'use strict';

    /*
     * @dictionary
     *
     * @translation
     */

    return ConfigView.extend({

        alias: 'maps',

        fakeMap: undefined,

        events: ConfigView.events,

        dict: dict,

        template: template,

        templateError: templateError,

        templateKeyInUse: templateKeyInUse,

        onApply: function () {
            // VALIDATION
            // https://jira.epages.de/browse/EPG-26271
            var invalidEntries = this.$('form').find('input:invalid'),
                $form;

            if (invalidEntries.length) {
                if (!this.fakesubmit) {
                    $form = this.$('form');
                    this.fakesubmit = $('<button type="submit">').addClass('HideElement').attr('form', $form.attr('id')).appendTo($form);
                }
                this.fakesubmit.trigger('click');
                return false;
            }

            return true;
        },

        onRadio: function (e) {
            var target = e.target,
                self = this;

            if (target.value !== self.gOptions[target.name]) {
                if (target.value !== 'klicktel') {
//#JSCOVERAGE_IF 0
                    // Set *self.gOptions* according to newly chosen type (i.e. *target.value*) and
                    // show fieldset with name *target.value*, hide the others.
                    self.gOptions = {};
                    self.gOptions[target.name] = target.value;
                    self.$el.find('fieldset').not('[name="' + target.name + '"], [name="' + target.value + '"]').attr('disabled', true).addClass('HideElement');
                    self.$el.find('fieldset[name="' + target.value + '"]').attr('disabled', false).removeClass('HideElement');
                    // Set remaining *self.gOptions* according to fieldset shown.
                    self.$el.find('fieldset[name="' + target.value + '"] :input').not('[name="address"]').filter(function () {
                        return this.value !== '';
                    }).trigger('change', true);
                    self.renderPreview();
//#JSCOVERAGE_ENDIF
                } else {
                    // Set *self.gOptions* according to newly chosen type (i.e. *target.value*) and
                    // show fieldset with name *target.value*, hide the others.
                    self.gOptions = {};
                    self.gOptions[target.name] = target.value;
                    self.$el.find('fieldset').not('[name="' + target.name + '"], [name="' + target.value + '"]').attr('disabled', true).addClass('HideElement');
                    self.$el.find('fieldset[name="' + target.value + '"]').attr('disabled', false).removeClass('HideElement');
                    // Set remaining *self.gOptions* according to fieldset shown.
                    self.$el.find('fieldset[name="' + target.value + '"] :input').trigger('change', true);
                    self.renderPreview();
                }
            }
        },

        addressToLatLng: function (address, noPreview) {
            var self = this;

            self.geocoder.geocode({
                address: address
            }, $.proxy(function (result, status) {
                if (status === 'OK' && result && result[0] && result[0].geometry && result[0].geometry.location) {
                    delete this.gOptions.address;
                    delete this.gOptions.center;
                    this.setOptions({
                        lat: result[0].geometry.location.lat(),
                        lng: result[0].geometry.location.lng(),
                        address: address
                    });
                    if (!noPreview) {
                        this.renderPreview();
                    }
                }
            }, self));
        },

        capitalizeFirstLetter: function (string) {
            return string.charAt(0).toUpperCase() + string.slice(1);
        },

        renderPreview: function () {
            var self = this,
                preview = self.elements.preview,
                parent = preview.parent(),
                gadgetPath = CONSTANTS.gWidgetNamespace + self.capitalizeFirstLetter(self.alias);

            if (preview.data('ui' + self.capitalizeFirstLetter(gadgetPath))) {
                preview[gadgetPath]('destroy');
            }

            preview.remove();
            preview = self.elements.preview = $('<div class="PreviewNode">').appendTo(parent);
            preview[gadgetPath]($.extend({}, self.gOptions, {
                callback: function (type, map, marker) {
                    self.map = map;
                    self.marker = marker;
                    switch (type) {
                    case 'google':
                        marker.setDraggable(true);
                        google.maps.event.addListener(marker, 'dragend', $.proxy(function () {
                            var position = self.marker.getPosition(),
                                center = self.map.getCenter();

                            self.setOptions({
                                lat: position.lat(),
                                lng: position.lng(),
                                center: {
                                    lat: center.lat(),
                                    lng: center.lng()
                                }
                            });

                        }, self));

                        google.maps.event.addListener(map, 'dragend', $.proxy(function () {
                            var center = self.map.getCenter();

                            self.setOptions({
                                center: {
                                    lat: center.lat(),
                                    lng: center.lng()
                                }
                            });

                        }, self));

                        google.maps.event.addListener(map, 'zoom_changed', $.proxy(function () {
                            self.setOptions({
                                zoom: self.map.getZoom()
                            });

                        }, self));

                        google.maps.event.addListener(map, 'maptypeid_changed', $.proxy(function () {
                            self.setOptions({
                                mapType: self.map.getMapTypeId()
                            });

                        }, self));

                        break;

//#JSCOVERAGE_IF 0
                    case 'bing':
                        self.map.AttachEvent('onchangeview', $.proxy(function (e) {
                            var center = this.map.GetCenter();

                            this.setOptions({
                                mapType: e.mapStyle,
                                zoom: e.zoomLevel,
                                center: {
                                    lat: center.Latitude,
                                    lng: center.Longitude
                                }
                            });

                        }, self));
//#JSCOVERAGE_ENDIF
                        break;
                    }
                }
            }));
        },

        googleLoadCallback: function (self) {
            self.geocoder = new google.maps.Geocoder();
            self.$('input[name="address"]').each(function () {
                var autocomplete = new google.maps.places.Autocomplete(this);
                google.maps.event.addListener(autocomplete, 'place_changed', function () {
//#JSCOVERAGE_IF 0
                    var place = this.getPlace();
                    if (place.geometry && place.geometry.location) {
                        delete self.gOptions.address;
                        delete self.gOptions.center;
                        self.setOptions({
                            lat: place.geometry.location.lat(),
                            lng: place.geometry.location.lng()
                        });
                        self.renderPreview();
                    }
                    self.addressToLatLng(place.formatted_address);
                });
//#JSCOVERAGE_ENDIF
            });
            if (!self.gOptions.lat || !self.gOptions.lng) {
                if (ep.config.shopAddress) {
                    self.addressToLatLng(ep.config.shopAddress, true);
                    self.$('input[name="address"]').each(function () {
                        this.value = ep.config.shopAddress;
                    });
                }
            }
            self.renderPreview();
        },

        initFakeMap: function (self) {
            self.fakeMap = new google.maps.Map(self.$('#FakeMapWrapper').find('#fakeMap')[0], {
                center: {lat: -34.397, lng: 150.644},
                zoom: 8
            });
        },

        triggerRenderPreviewWithKey: function (apikey, renderErrorTemplate, renderKeyInUseTemplate, buttonClick) {
            var self = this,
                timestamp = Date.now().toString(),
                ajaxData = {};

            ep.ajax({
                url: 'https://www.google.com/jsapi?timestamp=' + timestamp,
                dataType: 'script',
                cache: false
            }).done(function (data, textStatus, jqXHR) {
                google.load('maps', '3', {
                    other_params: apikey !== "" ? 'libraries=places&key=' + apikey + "&timestamp=" + timestamp
                                                : 'libraries=places' + "&timestamp=" + timestamp,
                    callback: function () {
                        self.$('input[name="address"]').removeAttr('disabled');
                        self.destroyFakeMap();
                        self.destroyPreviewMap();
                        self.initFakeMap(self);
                        self.$('.PreviewNode').append('<i style="margin-left: 170px; margin-top: 120px;" id="mapsSpinner" class="fa fa-spinner fa-pulse fa-3x fa-fw"></i>');

                        var id = setInterval(function () {
                            if (self.$('#fakeMap').find('div').hasClass('gm-style')) {
                                if (ep.config.googleMapsAPIKey !== "" && typeof ep.config.googleMapsAPIKey === "string") {
                                    self.$('#GMMessageWrap').prepend(renderKeyInUseTemplate);
                                }
                                // save key to database
                                ajaxData = $.extend({}, {APIKeyData: apikey});
                                self.googleLoadCallback(self);
                                if (buttonClick === true) {
                                    self.$('#GMMessageWrap').find('div').remove('#keyUseWrapper');
                                    ep.ajax({
                                        url: '?',
                                        type: 'POST',
                                        dataType: 'json',
                                        cache: false,
                                        data: {
                                            'ChangeAction': 'JSONHandleGoogleMapsAPIKey',
                                            'Operation': 'save',
                                            'data': JSON.stringify(ajaxData)
                                        }
                                    }).done(function (data, textStatus, jqXHR) {
                                        // write key to MBO config object
                                        ep.config.googleMapsAPIKey = apikey;
                                            self.$('#GMMessageWrap').prepend(renderKeyInUseTemplate);
                                    });
                                }
                                self.destroyFakeMap();
                            } else {
                                self.destroyFakeMap();
                                self.$('#GMMessageWrap').find('div').remove('#keyUseWrapper');
                                self.$('#GMMessageWrap').prepend(renderErrorTemplate);
                                self.$('input[name="address"]').attr('disabled', true);
                                self.$('input[name="googleMapsAPIKey"]').val('');

                                if(self.$('#KeyIconToggle').hasClass("fa-plus-circle")) {
                                    self.$('#KeyIconToggle').removeClass("fa-plus-circle").addClass("fa-minus-circle");
                                    self.$('#mapsToggleArea').slideToggle();
                                }
                            }
                            clearInterval(id);
                            self.$('.PreviewNode').children().remove('#mapsSpinner');
                        }, 3000);

                        var idGmErr = setInterval(function () {
                            if (self.$('.PreviewNode').find('div').hasClass('gm-err-container')) {
                                // delete "wrong" key from database
                                self.$('input[name="address"]').attr('disabled', true);
                                ep.ajax({
                                    url: '?',
                                    type: 'POST',
                                    dataType: 'json',
                                    cache: false,
                                    data: {
                                        'ChangeAction': 'JSONHandleGoogleMapsAPIKey',
                                        'Operation': 'delete',
                                        'data': ''
                                    }
                                }).done(function () {
                                    // condition "true" needed for handling in maps.js
                                    ep.config.googleMapsAPIKey = true;
                                    self.$('#GMMessageWrap').find('div').remove('#keyUseWrapper');
                                    self.$('#GMMessageWrap').prepend(renderErrorTemplate);
                                    self.destroyFakeMap();
                                    self.$('input[name="googleMapsAPIKey"]').val('');
                                    if(self.$('#KeyIconToggle').hasClass("fa-plus-circle")) {
                                        self.$('#KeyIconToggle').removeClass("fa-plus-circle").addClass("fa-minus-circle");
                                        self.$('#mapsToggleArea').slideToggle();
                                    }
                                });
                                clearInterval(idGmErr);
                            }
                        }, 1000);
                    }
                });
            });
        },

        hideToggleArea: function () {
            var self = this;

            self.$('#mapsToggleArea').css({
                "display": "none"
            });
        },

        destroyFakeMap: function () {
            var self = this;

            self.$('#fakeMap').children().remove();
            self.$('#fakeMap').removeAttr('style', 'position: relative; background-color: rgb(229, 227, 223); overflow: hidden; height:300px');

        },

        destroyPreviewMap: function () {
            var self = this;

            self.$('.PreviewNode').children().remove();
            self.$('.PreviewNode').removeAttr('style', 'position: relative; background-color: rgb(229, 227, 223); overflow: hidden; height:300px');
        },

        loadMapWithEnteredAPIKeyOnClick: function (renderErrorTemplate, renderKeyInUseTemplate) {
            var self = this;

            self.$('#SaveButtonAPIKey').on('click', function () {
                var apikeyForPreview,
                    buttonClick = true,
                    reg;

                self.$('input[name="address"]').removeClass("gm-err-autocomplete").removeAttr("placeholder disabled style");

                apikeyForPreview = typeof self.$('input[name="googleMapsAPIKey"]').val() === "string" ? self.$('input[name="googleMapsAPIKey"]').val() : "";

                if (apikeyForPreview.length >= 32 && apikeyForPreview.length <= 45) {
                    self.triggerRenderPreviewWithKey(apikeyForPreview, renderErrorTemplate, renderKeyInUseTemplate, buttonClick);

                } else {
                    self.destroyFakeMap();
                    self.destroyPreviewMap();
                    self.$('input[name="address"]').attr('disabled', true);
                    self.$('#GMMessageWrap').find('div').remove('#keyUseWrapper');
                    self.$('#GMMessageWrap').prepend(renderErrorTemplate);

                    if(self.$('#KeyIconToggle').hasClass("fa-plus-circle")) {
                        self.$('#KeyIconToggle').removeClass("fa-plus-circle").addClass("fa-minus-circle");
                        self.$('#mapsToggleArea').slideToggle();
                    }
                }
            });
        },

        togglePlusMinus: function () {
            var self = this;

            self.$('#KeyIconToggle').click(function () {
                var $this = $(this);

                self.$('#mapsToggleArea').slideToggle();
                if ($this.hasClass("fa-plus-circle")) {
                    $this.removeClass("fa-plus-circle").addClass("fa-minus-circle");
                } else {
                    $this.removeClass("fa-minus-circle").addClass("fa-plus-circle");
                }
            });
        },

        render: function () {
            var self = this,
                renderedTemplate,
                renderErrorTemplate,
                renderKeyInUseTemplate,
                types = [],
                defaults,
                apikey,
                buttonClick = false,
                i;

            if (self.differTypes) {
                // Merge *self.gOptions* in *types* for templating. (See below.)
                for (i = 0; i < self.gDefaults.length; i = i + 1) {
                    defaults = self.gDefaults[i];
                    types.push(defaults.type === self.gOptions.type ? self.gOptions : defaults);
                }
            }

            renderedTemplate = self.template(self.differTypes ? {
                'types': types,
                'gOptions': self.gOptions
            } : self.gOptions).dictParse(dict, true);

            renderedTemplate.find(':input').each(function () {
                if (this.placeholder) {
                    this.placeholder = dict.translate(this.placeholder);
                }
                $(this).uiInput();
            });

            // generate Error template to be appended in case of map loading error, because of wrong API key
            renderErrorTemplate = self.templateError().dictParse(dict, true);
            renderKeyInUseTemplate = self.templateKeyInUse().dictParse(dict, true);

            self.$el.append(renderedTemplate);
            self.elements = renderedTemplate.tmplItem('elements'); // preview element z.B.
            self.$('input[name="address"]').removeClass("gm-err-autocomplete").removeAttr("placeholder disabled style");
            self.hideToggleArea();
            self.togglePlusMinus();

            apikey = typeof self.$('input[name="googleMapsAPIKey"]').val() === "string" ? self.$('input[name="googleMapsAPIKey"]').val() : "";

            if (apikey === "true") {
                apikey = '';
            }

            self.triggerRenderPreviewWithKey(apikey, renderErrorTemplate, renderKeyInUseTemplate, buttonClick);
            if (ep.config.googleMapsAPIKey !== "" && typeof ep.config.googleMapsAPIKey === "string") {
                self.$('input[name="googleMapsAPIKey"]').val(ep.config.googleMapsAPIKey);
            } else {
                self.$('input[name="googleMapsAPIKey"]').val('');
            }
            self.loadMapWithEnteredAPIKeyOnClick(renderErrorTemplate, renderKeyInUseTemplate);

            return this;
        }
    });
});

